#include "CSunburstDimCluster.H"
#include "XmdvTool.H"
#include "stdio.h"
#include <math.h>
#include <string.h>

CSunburstDimCluster::CSunburstDimCluster() : CSunburstCluster()
{
	dim_name = NULL;
    weight = 1;
}


CSunburstDimCluster::~CSunburstDimCluster()
{
    if ( dim_name!= NULL )
        delete [] dim_name;
//    ~CSunburstCluster();
}

void CSunburstDimCluster::ResetNames()
{
    if ( okc_id >= 0 )
        return;

    char temp_name[300];
    strcpy(temp_name, "");
    CSunburstDimCluster *child = (CSunburstDimCluster *)children;
    if ( child != NULL )
    {
        child->ResetNames();
        strcat(temp_name, child->dim_name);
        child = (CSunburstDimCluster *)child->siblings;

        while ( child != NULL )
        {
            child->ResetNames();
            strcat(temp_name, "_");
            strcat(temp_name, child->dim_name);
            if ( strlen(temp_name) > 100 )
                break;
            child = (CSunburstDimCluster *)child->siblings;
        }
    }
    dim_name = new char[strlen(temp_name)+1];
    strcpy(dim_name, temp_name);
    return;
}


double CSunburstDimCluster::Calculate(ARR<double> all_data)
{
    if ( okc_id >= 0 )
        return all_data[okc_id];
    else
    {
        double weight = 0;
        double sum = 0, temp;
        CSunburstDimCluster *child = (CSunburstDimCluster *)children;
        while ( child != NULL )
        {
            weight += child->weight;
            temp = child->Calculate(all_data);
            if ( child->okc_id >= 0 )
            {
                 temp = (temp - dim_reducer->dim_min[child->okc_id])/(dim_reducer->dim_max[child->okc_id] - 
                         dim_reducer->dim_min[child->okc_id]);
            }
            sum += temp;
            child = (CSunburstDimCluster *)child->siblings;
        }
        if ( fabs(weight) < 1e-5 )
            return 0;
        else
            return sum/weight;
    }
}

void CSunburstDimCluster::CalculateWithDimClusterExtends(ARR<double> all_data, double &mean, double &max, double &min)
{
    if ( okc_id >= 0 )
    {
        mean = max = min = all_data[okc_id];
        return;
    }
    else
    {
        double total_weight = 0;
        double sum = 0;
        double child_mean, child_max, child_min, total_max, total_min;
        bool first = true;
        CSunburstDimCluster *child = (CSunburstDimCluster *)children;
        while ( child != NULL )
        {
            total_weight += child->weight;
            child->CalculateWithDimClusterExtends(all_data, child_mean, child_max, child_min);
            if ( child->okc_id >= 0 )
            {
                 child_mean = (child_mean - dim_reducer->dim_min[child->okc_id])/(dim_reducer->dim_max[child->okc_id] - 
                         dim_reducer->dim_min[child->okc_id]);
                 child_max = (child_max - dim_reducer->dim_min[child->okc_id])/(dim_reducer->dim_max[child->okc_id] - 
                         dim_reducer->dim_min[child->okc_id]);
                 child_min = (child_min - dim_reducer->dim_min[child->okc_id])/(dim_reducer->dim_max[child->okc_id] - 
                         dim_reducer->dim_min[child->okc_id]);
            }
            sum += child_mean * child->weight;
            if ( first )
            {
                total_max = child_max;
                total_min = child_min;
                first = false;
            }
            else
            {
                if ( total_max < child_max )
                    total_max = child_max;
                if ( total_min > child_min )
                    total_min = child_min;
            }
            child = (CSunburstDimCluster *)child->siblings;
        }
        if ( fabs(total_weight) < 1e-5 )
            mean = 0;
        else
            mean = sum/total_weight;

        max = total_max;
        min = total_min;
/*        if ( max > 1 )
            max = 1;
        if ( min < 0 )
            min = 0;
*/    }
}

int CSunburstDimCluster::Calculate(ARR<int> all_data)
{
    if ( okc_id >= 0 )
        return all_data[okc_id];
    else
    {
        double weight = 0;
        double sum = 0;
        CSunburstDimCluster *child = (CSunburstDimCluster *)children;
        while ( child != NULL )
        {
            weight += child->weight;
            sum += (child->Calculate(all_data)) * (child->weight);
            child = (CSunburstDimCluster *)child->siblings;
        }
        if ( fabs(weight) < 1e-5 )
            return 0;
        else
            return (int)(sum/weight);
    }
}


void CSunburstDimCluster::SizeStructureSelect(int threshold, bool mark )
{
	if ( entries <= threshold || entries == 1 )
    {
        ChangeSelectedStatus(mark);
    }
    else
    {
        CSunburstDimCluster *child = (CSunburstDimCluster *)children;

	    while ( child != NULL )
        {
		    child->SizeStructureSelect(threshold, mark );
		    child = (CSunburstDimCluster *)(child->siblings);
        }
    }
}


void CSunburstDimCluster::RadiusStructureSelect(double threshold, bool mark )
{
	if ( radius <= threshold || fabs(radius) < 1e-5 )
    {
        ChangeSelectedStatus(mark);
    }
    else
    {
        CSunburstDimCluster *child = (CSunburstDimCluster *)children;

	    while ( child != NULL )
        {
		    child->RadiusStructureSelect(threshold, mark );
		    child = (CSunburstDimCluster *)(child->siblings);
        }
    }
}


void CSunburstDimCluster::ChangeSelectedStatus(bool mark)
{
	if ( this->selected != mark )
    {
        if ( this->selected == true )
        {
            dim_reducer->DeleteRepresentativeDim(this);
        }
        else
        {
            dim_reducer->AddRepresentativeDim(this);
        }

        this->selected = mark;
    }
}

//reset selected mark;
//mark: the new selected mark to be set;
void CSunburstDimCluster::ResetSelected(bool mark)
{
    ChangeSelectedStatus(mark);

    CSunburstDimCluster *child = (CSunburstDimCluster *)children;

	while ( child != NULL )
    {
		child->ResetSelected(mark);
		child = (CSunburstDimCluster *)(child->siblings);
    }
}



